@@ -1,31 +1,62 @@
 import logging
+import ntpath
 import os
-import requests
+import shutil
 from functools import partial
+from tempfile import NamedTemporaryFile
 
+import requests
 from django.apps.registry import AppRegistryNotReady
+from django.core.files.uploadedfile import InMemoryUploadedFile
+from django.core.files.uploadedfile import UploadedFile
 from django.core.management import call_command
+from django.core.management.base import CommandError
 from django.http.response import Http404
+from django.http.response import HttpResponseBadRequest
+from django.utils.translation import get_language_from_request
 from django.utils.translation import gettext_lazy as _
+from morango.models import ScopeDefinition
+from morango.sync.controller import MorangoProfileController
+from requests.exceptions import HTTPError
+from rest_framework import decorators
 from rest_framework import serializers
+from rest_framework import status
 from rest_framework import viewsets
-from rest_framework.decorators import list_route
+from rest_framework.exceptions import APIException
+from rest_framework.exceptions import AuthenticationFailed
+from rest_framework.exceptions import ParseError
+from rest_framework.exceptions import PermissionDenied
+from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response
 from six import string_types
 
+from .permissions import FacilitySyncPermissions
+from kolibri.core.auth.constants.morango_sync import PROFILE_FACILITY_DATA
+from kolibri.core.auth.constants.morango_sync import State as FacilitySyncState
+from kolibri.core.auth.management.utils import get_client_and_server_certs
+from kolibri.core.auth.management.utils import get_dataset_id
+from kolibri.core.auth.models import Facility
 from kolibri.core.content.models import ChannelMetadata
 from kolibri.core.content.permissions import CanExportLogs
+from kolibri.core.content.permissions import CanImportUsers
 from kolibri.core.content.permissions import CanManageContent
 from kolibri.core.content.utils.channels import get_mounted_drive_by_id
 from kolibri.core.content.utils.channels import get_mounted_drives_with_channel_info
 from kolibri.core.content.utils.channels import read_channel_metadata_from_db_file
-from kolibri.core.content.utils.paths import get_content_database_file_path
 from kolibri.core.content.utils.paths import get_channel_lookup_url
+from kolibri.core.content.utils.paths import get_content_database_file_path
 from kolibri.core.content.utils.upgrade import diff_stats
+from kolibri.core.device.permissions import IsSuperuser
+from kolibri.core.device.permissions import NotProvisionedCanPost
 from kolibri.core.discovery.models import NetworkLocation
+from kolibri.core.discovery.utils.network.client import NetworkClient
+from kolibri.core.discovery.utils.network.errors import NetworkLocationNotFound
+from kolibri.core.discovery.utils.network.errors import URLParseError
+from kolibri.core.logger.csv_export import CSV_EXPORT_FILENAMES
 from kolibri.core.tasks.exceptions import JobNotFound
 from kolibri.core.tasks.exceptions import UserCancelledError
 from kolibri.core.tasks.job import State
+from kolibri.core.tasks.main import facility_queue
 from kolibri.core.tasks.main import priority_queue
 from kolibri.core.tasks.main import queue
 from kolibri.core.tasks.utils import get_current_job
@@ -146,21 +177,31 @@ def validate_deletion_task(request, task_description):
     return task
 
 
-class TasksViewSet(viewsets.ViewSet):
-    def get_permissions(self):
+class BaseViewSet(viewsets.ViewSet):
+    queues = []
+    permission_classes = []
+
+    def initial(self, request, *args, **kwargs):
+        if len(self.permission_classes) == 0:
+            self.permission_classes = self.default_permission_classes()
+        return super(BaseViewSet, self).initial(request, *args, **kwargs)
+
+    def default_permission_classes(self):
         # task permissions shared between facility management and device management
         if self.action in ["list", "deletefinishedtasks"]:
-            permission_classes = [CanManageContent | CanExportLogs]
-        # exclusive permission for facility management
+            return [CanManageContent | CanExportLogs]
         elif self.action == "startexportlogcsv":
-            permission_classes = [CanExportLogs]
+            return [CanExportLogs]
+        elif self.action in ["importusersfromcsv", "exportuserstocsv"]:
+            return [CanImportUsers]
+
         # this was the default before, so leave as is for any other endpoints
-        else:
-            permission_classes = [CanManageContent]
-        return [permission() for permission in permission_classes]
+        return [CanManageContent]
 
     def list(self, request):
-        jobs_response = [_job_to_response(j) for j in queue.jobs + priority_queue.jobs]
+        jobs_response = [
+            _job_to_response(j) for _queue in self.queues for j in _queue.jobs
+        ]
 
         return Response(jobs_response)
 
@@ -169,20 +210,90 @@ class TasksViewSet(viewsets.ViewSet):
         pass
 
     def retrieve(self, request, pk=None):
-        try:
-            task = _job_to_response(queue.fetch_job(pk))
-            return Response(task)
-        except JobNotFound:
+        for _queue in self.queues:
             try:
-                task = _job_to_response(priority_queue.fetch_job(pk))
+                task = _job_to_response(_queue.fetch_job(pk))
+                break
             except JobNotFound:
-                raise Http404("Task with {pk} not found".format(pk=pk))
+                continue
+        else:
+            raise Http404("Task with {pk} not found".format(pk=pk))
+
+        return Response(task)
 
     def destroy(self, request, pk=None):
         # unimplemented for now.
         pass
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
+    def canceltask(self, request):
+        """
+        Cancel a task with its task id given in the task_id parameter.
+        """
+
+        if "task_id" not in request.data:
+            raise serializers.ValidationError("The 'task_id' field is required.")
+        if not isinstance(request.data["task_id"], string_types):
+            raise serializers.ValidationError("The 'task_id' should be a string.")
+
+        for _queue in self.queues:
+            try:
+                _queue.cancel(request.data["task_id"])
+                break
+            except JobNotFound:
+                continue
+
+        return Response({})
+
+    @decorators.action(methods=["post"], detail=False)
+    def cleartasks(self, request):
+        """
+        Cancels all running tasks.
+        """
+
+        for _queue in self.queues:
+            _queue.empty()
+
+        return Response({})
+
+    @decorators.action(methods=["post"], detail=False)
+    def cleartask(self, request):
+        # Given a single task ID, clear it from the queue
+        task_id = request.data.get("task_id")
+        if not task_id:
+            return Response({})
+
+        for _queue in self.queues:
+            _queue.clear_job(task_id)
+
+        return Response({"task_id": task_id})
+
+    @decorators.action(methods=["post"], detail=False)
+    def deletefinishedtasks(self, request):
+        """
+        Delete all tasks that have succeeded, failed, or been cancelled.
+        """
+        task_id = request.data.get("task_id")
+        if task_id:
+            for _queue in self.queues:
+                _queue.clear_job(task_id)
+        else:
+            for _queue in self.queues:
+                _queue.clear()
+        return Response({})
+
+
+class TasksViewSet(BaseViewSet):
+    queues = [queue, priority_queue]
+
+    def default_permission_classes(self):
+        # exclusive permission for facility management
+        if self.action == "startexportlogcsv":
+            return [CanExportLogs]
+
+        return super(TasksViewSet, self).permission_classes
+
+    @decorators.action(methods=["post"], detail=False)
     def startchannelupdate(self, request):
 
         sourcetype = request.data.get("sourcetype", None)
@@ -223,7 +334,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startremotebulkimport(self, request):
         if not isinstance(request.data, list):
             raise serializers.ValidationError(
@@ -251,7 +362,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startremotechannelimport(self, request):
 
         task = validate_remote_import_task(request, request.data)
@@ -272,7 +383,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startremotecontentimport(self, request):
 
         task = validate_remote_import_task(request, request.data)
@@ -296,7 +407,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdiskbulkimport(self, request):
         if not isinstance(request.data, list):
             raise serializers.ValidationError(
@@ -324,7 +435,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdiskchannelimport(self, request):
         task = validate_local_import_task(request, request.data)
 
@@ -344,7 +455,7 @@ class TasksViewSet(viewsets.ViewSet):
         resp = _job_to_response(priority_queue.fetch_job(job_id))
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdiskcontentimport(self, request):
         task = validate_local_import_task(request, request.data)
 
@@ -368,7 +479,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startbulkdelete(self, request):
         if not isinstance(request.data, list):
             raise serializers.ValidationError(
@@ -397,7 +508,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdeletechannel(self, request):
         """
         Delete a channel and all its associated content from the server
@@ -426,7 +537,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdiskbulkexport(self, request):
         if not isinstance(request.data, list):
             raise serializers.ValidationError(
@@ -453,7 +564,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def startdiskexport(self, request):
         """
         Export a channel to a local drive, and copy content to the drive.
@@ -479,87 +590,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
-    def startdataportalsync(self, request):
-        """
-        Initiate a PUSH sync with Kolibri Data Portal.
-        """
-        task = {
-            "facility": request.data["facility"],
-            "type": "SYNCDATAPORTAL",
-            "started_by": request.user.pk,
-        }
-
-        job_id = queue.enqueue(
-            call_command,
-            "sync",
-            facility=task["facility"],
-            noninteractive=True,
-            extra_metadata=task,
-            track_progress=False,
-            cancellable=False,
-        )
-        # attempt to get the created Task, otherwise return pending status
-        resp = _job_to_response(queue.fetch_job(job_id))
-
-        return Response(resp)
-
-    @list_route(methods=["post"])
-    def canceltask(self, request):
-        """
-        Cancel a task with its task id given in the task_id parameter.
-        """
-
-        if "task_id" not in request.data:
-            raise serializers.ValidationError("The 'task_id' field is required.")
-        if not isinstance(request.data["task_id"], string_types):
-            raise serializers.ValidationError("The 'task_id' should be a string.")
-        try:
-            queue.cancel(request.data["task_id"])
-        except JobNotFound:
-            try:
-                priority_queue.cancel(request.data["task_id"])
-            except JobNotFound:
-                pass
-
-        return Response({})
-
-    @list_route(methods=["post"])
-    def cleartasks(self, request):
-        """
-        Cancels all running tasks.
-        """
-
-        queue.empty()
-        priority_queue.empty()
-        return Response({})
-
-    @list_route(methods=["post"])
-    def cleartask(self, request):
-        # Given a single task ID, clear it from the queue
-        task_id = request.data.get("task_id")
-        if task_id:
-            queue.clear_job(task_id)
-            priority_queue.clear_job(task_id)
-            return Response({"task_id": task_id})
-        else:
-            return Response({})
-
-    @list_route(methods=["post"])
-    def deletefinishedtasks(self, request):
-        """
-        Delete all tasks that have succeeded, failed, or been cancelled.
-        """
-        task_id = request.data.get("task_id")
-        if task_id:
-            queue.clear_job(task_id)
-            priority_queue.clear_job(task_id)
-        else:
-            queue.clear()
-            priority_queue.clear()
-        return Response({})
-
-    @list_route(methods=["get"])
+    @decorators.action(methods=["get"], detail=False)
     def localdrive(self, request):
         drives = get_mounted_drives_with_channel_info()
 
@@ -569,24 +600,149 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(out)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
+    def importusersfromcsv(self, request):
+        """
+        Import users, classes, roles and roles assignemnts from a csv file.
+        :param: FILE: file dictionary with the file object
+        :param: csvfile: filename of the file stored in kolibri temp folder
+        :param: dryrun: validate the data but don't modify the database
+        :param: delete: Users not in the csv will be deleted from the facility, and classes cleared
+        :returns: An object with the job information
+        """
+
+        def manage_fileobject(request, temp_dir):
+            upload = UploadedFile(request.FILES["csvfile"])
+            # Django uses InMemoryUploadedFile for files less than 2.5Mb
+            # and TemporaryUploadedFile for bigger files:
+            if type(upload.file) == InMemoryUploadedFile:
+                with NamedTemporaryFile(
+                    dir=temp_dir, suffix=".upload", delete=False
+                ) as dest:
+                    filepath = dest.name
+                    for chunk in upload.file.chunks():
+                        dest.write(chunk)
+            else:
+                tmpfile = upload.file.temporary_file_path()
+                filename = ntpath.basename(tmpfile)
+                filepath = os.path.join(temp_dir, filename)
+                shutil.copy(tmpfile, filepath)
+            return filepath
+
+        temp_dir = os.path.join(conf.KOLIBRI_HOME, "temp")
+        if not os.path.isdir(temp_dir):
+            os.mkdir(temp_dir)
+
+        locale = get_language_from_request(request)
+        # the request must contain either an object file
+        # or the filename of the csv stored in Kolibri temp folder
+        # Validation will provide the file object, while
+        # Importing will provide the filename, previously validated
+        if not request.FILES:
+            filename = request.data.get("csvfile", None)
+            if filename:
+                filepath = os.path.join(temp_dir, filename)
+            else:
+                return HttpResponseBadRequest("The request must contain a file object")
+        else:
+            if "csvfile" not in request.FILES:
+                return HttpResponseBadRequest("Wrong file object")
+            filepath = manage_fileobject(request, temp_dir)
+
+        delete = request.data.get("delete", None)
+        dryrun = request.data.get("dryrun", None)
+        userid = request.user.pk
+        facility_id = request.data.get("facility_id", None)
+        job_type = "IMPORTUSERSFROMCSV"
+        job_metadata = {"type": job_type, "started_by": userid, "facility": facility_id}
+        job_args = ["bulkimportusers"]
+        if dryrun:
+            job_args.append("--dryrun")
+        if delete:
+            job_args.append("--delete")
+        job_args.append(filepath)
+
+        job_kwd_args = {
+            "facility": facility_id,
+            "userid": userid,
+            "locale": locale,
+            "extra_metadata": job_metadata,
+            "track_progress": True,
+        }
+
+        job_id = priority_queue.enqueue(call_command, *job_args, **job_kwd_args)
+
+        resp = _job_to_response(priority_queue.fetch_job(job_id))
+
+        return Response(resp)
+
+    @decorators.action(methods=["post"], detail=False)
+    def exportuserstocsv(self, request):
+        """
+        Export users, classes, roles and roles assignemnts to a csv file.
+
+        :param: facility_id
+        :returns: An object with the job information
+
+        """
+        facility_id = request.data.get("facility_id", None)
+
+        try:
+            if facility_id:
+                facility = Facility.objects.get(pk=facility_id).id
+            else:
+                facility = request.user.facility
+        except Facility.DoesNotExist:
+            raise serializers.ValidationError(
+                "Facility with ID {} does not exist".format(facility_id)
+            )
+
+        job_type = "EXPORTUSERSTOCSV"
+        job_metadata = {
+            "type": job_type,
+            "started_by": request.user.pk,
+            "facility": facility,
+        }
+        locale = get_language_from_request(request)
+
+        job_id = priority_queue.enqueue(
+            call_command,
+            "bulkexportusers",
+            facility=facility,
+            locale=locale,
+            overwrite="true",
+            extra_metadata=job_metadata,
+            track_progress=True,
+        )
+
+        resp = _job_to_response(priority_queue.fetch_job(job_id))
+
+        return Response(resp)
+
+    @decorators.action(methods=["post"], detail=False)
     def startexportlogcsv(self, request):
         """
         Dumps in csv format the required logs.
         By default it will be dump contentsummarylog.
 
         :param: logtype: Kind of log to dump, summary or session
+        :param: facility
         :returns: An object with the job information
 
         """
-        csv_export_filenames = {
-            "session": "content_session_logs.csv",
-            "summary": "content_summary_logs.csv",
-        }
+        facility_id = request.data.get("facility", None)
+        if facility_id:
+            facility = Facility.objects.get(pk=facility_id)
+        else:
+            facility = request.user.facility
+
         log_type = request.data.get("logtype", "summary")
-        if log_type in csv_export_filenames.keys():
+        if log_type in CSV_EXPORT_FILENAMES.keys():
             logs_dir = os.path.join(conf.KOLIBRI_HOME, "log_export")
-            filepath = os.path.join(logs_dir, csv_export_filenames[log_type])
+            filepath = os.path.join(
+                logs_dir,
+                CSV_EXPORT_FILENAMES[log_type].format(facility.name, facility_id[:4]),
+            )
         else:
             raise Http404(
                 "Impossible to create a csv export file for {}".format(log_type)
@@ -598,13 +754,18 @@ class TasksViewSet(viewsets.ViewSet):
             "EXPORTSUMMARYLOGCSV" if log_type == "summary" else "EXPORTSESSIONLOGCSV"
         )
 
-        job_metadata = {"type": job_type, "started_by": request.user.pk}
+        job_metadata = {
+            "type": job_type,
+            "started_by": request.user.pk,
+            "facility": facility.id,
+        }
 
         job_id = priority_queue.enqueue(
             call_command,
             "exportlogs",
             log_type=log_type,
             output_file=filepath,
+            facility=facility.id,
             overwrite="true",
             extra_metadata=job_metadata,
             track_progress=True,
@@ -614,7 +775,7 @@ class TasksViewSet(viewsets.ViewSet):
 
         return Response(resp)
 
-    @list_route(methods=["post"])
+    @decorators.action(methods=["post"], detail=False)
     def channeldiffstats(self, request):
         job_metadata = {}
         channel_id = request.data.get("channel_id")
@@ -677,6 +838,248 @@ class TasksViewSet(viewsets.ViewSet):
         return Response(resp)
 
 
+class FacilityTasksViewSet(BaseViewSet):
+    queues = [facility_queue]
+
+    def default_permission_classes(self):
+        permission_classes = super(FacilityTasksViewSet, self).permission_classes
+
+        if self.action in ["list", "retrieve"]:
+            return [p | FacilitySyncPermissions for p in permission_classes]
+
+        # All other permissions are deferred to permission_classes decorator
+        return []
+
+    @decorators.action(
+        methods=["post"], detail=False, permission_classes=[FacilitySyncPermissions]
+    )
+    def startdataportalsync(self, request):
+        """
+        Initiate a PUSH sync with Kolibri Data Portal.
+        """
+        job_data = validate_prepare_sync_job(
+            request, extra_metadata=prepare_sync_task(request, type="SYNCDATAPORTAL")
+        )
+        job_id = facility_queue.enqueue(call_command, "sync", **job_data)
+
+        resp = _job_to_response(facility_queue.fetch_job(job_id))
+        return Response(resp)
+
+    @decorators.action(methods=["post"], detail=False, permission_classes=[IsSuperuser])
+    def startdataportalbulksync(self, request):
+        """
+        Initiate a PUSH sync with Kolibri Data Portal for ALL registered facilities.
+        """
+        responses = []
+        facilities = Facility.objects.filter(dataset__registered=True).values_list(
+            "id", "name"
+        )
+
+        for id, name in facilities:
+            request.data.update(facility=id, facility_name=name)
+            responses.append(self.startdataportalsync(request).data)
+
+        return Response(responses)
+
+    # Method needs to be available in Setup Wizard as well
+    @decorators.action(
+        methods=["post"],
+        detail=False,
+        permission_classes=[IsSuperuser | NotProvisionedCanPost],
+    )
+    def startpeerfacilityimport(self, request):
+        """
+        Initiate a PULL of a specific facility from another device.
+        """
+        job_data = validate_and_prepare_peer_sync_job(
+            request,
+            no_push=True,
+            no_provision=True,
+            extra_metadata=prepare_sync_task(request, type="SYNCPEER/PULL"),
+        )
+
+        job_id = facility_queue.enqueue(call_command, "sync", **job_data)
+
+        resp = _job_to_response(facility_queue.fetch_job(job_id))
+        return Response(resp)
+
+    @decorators.action(
+        methods=["post"], detail=False, permission_classes=[FacilitySyncPermissions]
+    )
+    def startpeerfacilitysync(self, request):
+        """
+        Initiate a SYNC (PULL + PUSH) of a specific facility from another device.
+        """
+        job_data = validate_and_prepare_peer_sync_job(
+            request, extra_metadata=prepare_sync_task(request, type="SYNCPEER/FULL")
+        )
+        job_id = facility_queue.enqueue(call_command, "sync", **job_data)
+
+        resp = _job_to_response(facility_queue.fetch_job(job_id))
+        return Response(resp)
+
+    @decorators.action(methods=["post"], detail=False, permission_classes=[IsSuperuser])
+    def startdeletefacility(self, request):
+        """
+        Initiate a task to delete a facility
+        """
+        try:
+            facility_id = request.data.get("facility")
+            if not facility_id:
+                raise KeyError()
+        except KeyError:
+            raise ParseError(
+                dict(code="INVALID_FACILITY", message="Missing `facility` parameter")
+            )
+
+        if not Facility.objects.filter(id=facility_id).exists():
+            raise ValidationError(
+                dict(code="INVALID_FACILITY", message="Facility doesn't exist")
+            )
+
+        if not Facility.objects.exclude(id=facility_id).exists():
+            raise ValidationError(
+                dict(
+                    code="SOLE_FACILITY",
+                    message="Cannot delete the sole facility on the device",
+                )
+            )
+
+        if request.user.is_facility_user and request.user.facility_id == facility_id:
+            raise ValidationError(
+                dict(code="FACILITY_MEMBER", message="User is member of facility")
+            )
+
+        facility_name = Facility.objects.get(id=facility_id).name
+        job_id = facility_queue.enqueue(
+            call_command,
+            "deletefacility",
+            facility=facility_id,
+            track_progress=True,
+            noninteractive=True,
+            cancellable=False,
+            extra_metadata=dict(
+                facility=facility_id,
+                facility_name=facility_name,
+                started_by=request.user.pk,
+                started_by_username=request.user.username,
+                type="DELETEFACILITY",
+            ),
+        )
+
+        resp = _job_to_response(facility_queue.fetch_job(job_id))
+        return Response(resp)
+
+
+class ResourceGoneError(APIException):
+    """
+    API error for when a peer no longer is online
+    """
+
+    status_code = status.HTTP_410_GONE
+    default_detail = "Unable to connect"
+
+
+def prepare_sync_task(request, **kwargs):
+    facility_id = request.data.get("facility")
+    task_data = dict(
+        facility=facility_id,
+        started_by=request.user.pk,
+        started_by_username=request.user.username,
+        sync_state=FacilitySyncState.PENDING,
+        bytes_sent=0,
+        bytes_received=0,
+    )
+
+    task_type = kwargs.get("type")
+    if task_type in ["SYNCPEER/PULL", "SYNCPEER/FULL"]:
+        # Extra metadata that can be passed from the client
+        extra_task_data = dict(
+            facility_name=request.data.get("facility_name", ""),
+            device_name=request.data.get("device_name", ""),
+            device_id=request.data.get("device_id", ""),
+            baseurl=request.data.get("baseurl", ""),
+        )
+        task_data.update(extra_task_data)
+    elif task_type == "SYNCDATAPORTAL":
+        # Extra metadata that can be passed from the client
+        extra_task_data = dict(facility_name=request.data.get("facility_name", ""))
+        task_data.update(extra_task_data)
+
+    task_data.update(kwargs)
+    return task_data
+
+
+def validate_prepare_sync_job(request, **kwargs):
+    # ensure we have the facility
+    try:
+        facility_id = request.data.get("facility")
+        if not facility_id:
+            raise KeyError()
+    except KeyError:
+        raise ParseError("Missing `facility` parameter")
+
+    job_data = dict(
+        facility=facility_id,
+        chunk_size=200,
+        noninteractive=True,
+        extra_metadata=dict(),
+        track_progress=True,
+        cancellable=False,
+    )
+
+    job_data.update(kwargs)
+    return job_data
+
+
+def validate_and_prepare_peer_sync_job(request, **kwargs):
+    # validate the baseurl
+    try:
+        address = request.data.get("baseurl")
+        if not address:
+            raise KeyError()
+
+        baseurl = NetworkClient(address=address).base_url
+    except KeyError:
+        raise ParseError("Missing `baseurl` parameter")
+    except URLParseError:
+        raise ParseError("Invalid URL")
+    except NetworkLocationNotFound:
+        raise ResourceGoneError()
+
+    job_data = validate_prepare_sync_job(request, baseurl=baseurl, **kwargs)
+
+    facility_id = job_data.get("facility")
+    username = request.data.get("username", None)
+    password = request.data.get("password", None)
+
+    # call this in case user directly syncs without migrating database
+    if not ScopeDefinition.objects.filter():
+        call_command("loaddata", "scopedefinitions")
+
+    controller = MorangoProfileController(PROFILE_FACILITY_DATA)
+    network_connection = controller.create_network_connection(baseurl)
+
+    # try to get the certificate, which will save it if successful
+    try:
+        # make sure we get the dataset ID
+        dataset_id = get_dataset_id(
+            baseurl, identifier=facility_id, noninteractive=True
+        )
+
+        # username and password are not required for this to succeed unless there is no cert
+        get_client_and_server_certs(
+            username, password, dataset_id, network_connection, noninteractive=True
+        )
+    except (CommandError, HTTPError) as e:
+        if not username and not password:
+            raise PermissionDenied()
+        else:
+            raise AuthenticationFailed(e)
+
+    return job_data
+
+
 def _remoteimport(
     channel_id,
     baseurl,
@@ -712,21 +1115,18 @@ def _remoteimport(
 
     job.save_meta()
 
-    # Skip importcontent step if updating and no nodes have changed
-    if is_updating and (node_ids is not None) and len(node_ids) == 0:
-        pass
-    else:
-        call_command(
-            "importcontent",
-            "network",
-            channel_id,
-            baseurl=baseurl,
-            peer_id=peer_id,
-            node_ids=node_ids,
-            exclude_node_ids=exclude_node_ids,
-            update_progress=update_progress,
-            check_for_cancel=check_for_cancel,
-        )
+    call_command(
+        "importcontent",
+        "network",
+        channel_id,
+        baseurl=baseurl,
+        peer_id=peer_id,
+        node_ids=node_ids,
+        exclude_node_ids=exclude_node_ids,
+        import_updates=is_updating,
+        update_progress=update_progress,
+        check_for_cancel=check_for_cancel,
+    )
 
 
 def _diskimport(
